/*
 * gnote
 *
 * Copyright (C) 2012,2017,2020 Aurimas Cernius
 * Copyright (C) 2009 Hubert Figuiere
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */




#ifndef __SHARP_DATETIME_HPP__
#define __SHARP_DATETIME_HPP__

#include <time.h>

#include <glibmm/datetime.h>
#include <glibmm/ustring.h>
#include <glibmm/timeval.h>

#include "timespan.hpp"

namespace sharp {

Glib::ustring date_time_to_string(const Glib::DateTime & dt, const char *format);
Glib::ustring date_time_to_string(const Glib::DateTime & dt, const Glib::ustring & format);
Glib::ustring date_time_to_iso8601(const Glib::DateTime & dt);
Glib::DateTime date_time_from_iso8601(const Glib::ustring & dt);

}


bool operator==(const Glib::DateTime & x, const Glib::DateTime & y);
bool operator!=(const Glib::DateTime & x, const Glib::DateTime & y);
bool operator<(const Glib::DateTime & x, const Glib::DateTime & y);
bool operator<=(const Glib::DateTime & x, const Glib::DateTime & y);
bool operator>(const Glib::DateTime & x, const Glib::DateTime & y);
bool operator>=(const Glib::DateTime & x, const Glib::DateTime & y);

#endif
